-- Word - a very derivative word-guessing game
-- Copyright (C) 2023 Duane Robertson

-- main.lua

-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.


local FONT_FILE = 'art/roboto/Roboto-Bold.ttf'
local FSIZE = 100
local IHEIGHT = 1000
local IWIDTH = 720

local COLORS = {
	{0.9, 0.9, 1}, {1, 1, 0}, {0, 1, 0}
}

local android
local canv
local font
local font_small
local game_over
local guess = 1
local guesses = {}
local matches = {}
local scale = { x=1, y=1 }
local word
local word_count = 0
local words = {}


local function end_game()
	game_over = true
	love.keyboard.setTextInput(false)
end


local function cheap_random(seed, range)
	local num = (7^5 * seed) % (2^31 - 1)
	if range then
		num = (num % range) + 1
	end
	return num
end

if false then
	local dist = {}
	for i = 1, 100 do
		dist[cheap_random(i, 100)] = (dist[cheap_random(i, 100)] or 0) + 1
		print(cheap_random(i, 100))
	end
	for i = 1, 100 do
		print(i, dist[i] or 0)
	end
end


local function load_words()
	local word_picks = {}
	for lin in love.filesystem.lines('words.txt') do
		words[lin:upper()] = true
		word_picks[#word_picks+1] = lin
		word_count = word_count + 1
	end

	-- new word every hour
	local seed = math.floor(os.time() / 3600)
	local num = cheap_random(seed, word_count)

	word = word_picks[num]:upper()
end


local function rate_guess()
	if not words[guesses[guess]] then
		guesses[guess] = ''
		return
	end

	for i = 1, 5 do
		local w = guesses[guess]
		if w:sub(i, i) == word:sub(i, i) then
			matches[guess][i] = 3
		elseif word:find(w:sub(i, i)) then
			matches[guess][i] = 2
		else
			matches[guess][i] = 1
		end
	end

	if guess == 5 or word == guesses[guess] then
		end_game()
	end

	guess = guess + 1
end


function love.draw()
	love.graphics.setCanvas(canv)
	love.graphics.clear()

	love.graphics.setFont(font_small)

	love.graphics.setFont(font)

	for j = 1, 5 do
		local w = guesses[j] or ''
		for i = 1, 5 do
			love.graphics.setColor(0.3, 0.3, 0.3)
			love.graphics.rectangle('fill', i * 100 + 15, j * 100 + 105, 90, 90)
			love.graphics.setColor(COLORS[matches[j][i]])
			love.graphics.printf(w:sub(i, i), i * 100 + 10, j * 100 + 95, 100, 'center')
			love.graphics.setColor(1, 1, 1)
		end
	end

	love.graphics.setCanvas()

	if android then
		love.graphics.draw(canv, 0, 0, 0, scale.x, scale.x)
	else
		love.graphics.draw(canv)
	end
end


function love.keypressed(key)
	if key == 'backspace' then
		guesses[guess] = (guesses[guess] or ''):sub(1, -2)
	elseif key == 'return' then
		rate_guess()
	end
end


function love.load()
	local dx, dy = love.graphics.getDimensions()

	canv = love.graphics.newCanvas(IWIDTH, IHEIGHT, { dpiscale = 1 })
	font = love.graphics.newFont(FONT_FILE, FSIZE)
	font_small = love.graphics.newFont(36)

	for i = 1, 5 do
		matches[i] = {1,1,1,1,1}
	end

	love.keyboard.setKeyRepeat(true)

	android = (love.system.getOS() == 'Android')

	if android then
		local desk_x = math.min(love.window.getDesktopDimensions())
		local safe = dy / dx

		dx, dy = desk_x, math.floor(desk_x * safe)
		scale.x = dx / IWIDTH
		scale.y = dy / IHEIGHT
	end

	load_words()
end


function love.mousepressed(x, y, button)
    if button == 1 and not game_over then
		love.keyboard.setTextInput( true )
	elseif button == 2 then
		-- nop
    end
end


function love.textinput(text)
	if text:find('^%a$') then
		guesses[guess] = (guesses[guess] or '') .. text:upper()
		guesses[guess] = guesses[guess]:sub(1, 5)
	end
end
